package quotes;

import java.util.List;

/**
 * @author delpiano
 *
 * ChannelC is the configuration interface of the Channel Abstraction.
 * These two interfaces have been separated to stress on the
 * difference between fundtional behaviours (and consequently methods)
 * and configuration matters. They could as well have been merged.
 */
public interface ChannelC extends Channel {
	
	/** Returns the name of the channel.
	 * @return name the name of the channel.
	 */
	String getName();
	
	/** Sets the name of the channel.
	 * @param name the name of the channel.
	 */
	void setName(String name);
	
	/** Adds a new subscriber interested by the topic vehicled by this channel.
	 * @param subscriber the new subcriber that should be added.
	 */
	void addSubscriber(Subscriber subscriber);
	
	/** Removes a subscriber that is no more interested by the topic vehicled by this channel.
	 * @param subscriber the subcriber that should be removed from the list of interested subscribers.
	 */
	void removeSubscriber(Subscriber subscriber);
	
	/** Returns a list of all the subscribers to this channel.
	 * In fact, it returns a copy of the list of subscribers, to
	 * avoid concurrency/sharing problems.
	 * @return a copy of the subscribers to this channel.
	 */
	List getSubscribers();

}
