package quotes.local;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Observable;

import quotes.Channel;
import quotes.Subscriber;

/**
 * @author delpiano
 *
 * A Subscriber that stores the received messages in a fixed-size buffer, used in round-robin mode.
 */
public class BufferedSubscriber extends Observable implements Subscriber {
	
	private int number_of_received_messages;
	private Hashtable channels_buffers = new Hashtable();
	private Hashtable channels_buffers_current_slot_index = new Hashtable();
	private int buffer_size;

	/** Creates a buffered subcriber with a given size.
	 * @param buffer_size the buffer size.
	 */
	public BufferedSubscriber(int buffer_size) {
		super();
		this.buffer_size = buffer_size;
	}
	
	/** Returns the number of received messages.
	 * @return number of received messages.
	 */
	public int getNumberOfReceivedMessages() { return number_of_received_messages; }

	/**
	 * @see quotes.Subscriber#receiveMessage(Channel, Object, Object)
	 */
	public void receiveMessage(
		Channel received_from_channel,
		Object message_source,
		Object message) {
			
			number_of_received_messages++;
						
			Object[] q = (Object[]) channels_buffers.get(received_from_channel);
			
			if (q == null) {
				q = new Object[buffer_size];
				channels_buffers.put(received_from_channel , q);
				channels_buffers_current_slot_index.put(received_from_channel , new Integer(0));
			}
			
			int crrnt_slt = ((Integer) channels_buffers_current_slot_index.get(received_from_channel)).intValue();
			
			q[crrnt_slt] = message;
			
			crrnt_slt = (crrnt_slt + 1) % buffer_size;
			
			channels_buffers_current_slot_index.put(received_from_channel , new Integer(crrnt_slt));
			
			setChanged();
			notifyObservers("toto");
			
	}
	
	/** Returns an enumeration of all the channels that have published information on this Subscriber.
	 * @return the enumeration of all the channels that have published information on this Subscriber.
	 */
	public Enumeration getChannels() { return channels_buffers.keys(); }
	
	/** Returns the index of the last message received on a given Channel. Returns -1 if none has ever been received.
	 * @param channel the buffered channel,
	 * @return the index.
	 */
	public int getIndexOfLatestReceivedValueOnChannel(Channel channel) {
		Object vl = channels_buffers_current_slot_index.get(channel);
		if (vl == null) { return -1; }
		return ((Integer) vl).intValue(); 
	}
	
	/** Returns the value received on a given channel, at a given index. Returns <code>null</code> is nothing has been received.
	 * @param channel the channel,
	 * @param index the index.
	 * @return the buffered value.
	 */
	public Object getBufferedValue(Channel channel, int index) {
		Object vl = channels_buffers.get(channel);
		if (vl == null) { return null; }
		if ((index < 0) || (index >= buffer_size)) { return null; }
		return ((Object[]) vl)[index];
	}

	/**
	 * Returns the buffer_size.
	 * @return int
	 */
	public int getBufferSize() {
		return buffer_size;
	}
	
}
